import {Component, Input, OnDestroy, OnInit} from '@angular/core';
import {QueueSetting, WorkflowSetting} from "../../model/setting";
import {faPlus, faTimes} from "@fortawesome/free-solid-svg-icons";
import {SettingsService} from "../../services/settings.service";
import {ConfigurationService} from "../../env/configuration.service";
import {AlertService} from "../../services/alert.service";
import {Subscription} from "rxjs";
import {FormControl, FormGroup, Validators} from "@angular/forms";


@Component({
  selector: 'app-queue-settings',
  templateUrl: './queue-settings.component.html',
  styleUrls: ['./queue-settings.component.scss']
})
export class QueueSettingsComponent implements OnInit, OnDestroy {

  @Input() queueSetting: QueueSetting;
  private workflowSettings$: Subscription;
  public workflowSettings: Array<WorkflowSetting>;

  public faPlus = faPlus;
  public faTimes = faTimes;

  public configUrl: string;

  public newFileForm: FormGroup;

  constructor(
    private settingService: SettingsService,
    private configService: ConfigurationService,
    private alertService: AlertService
  ) {
    this.newFileForm = new FormGroup({
      name: new FormControl(null, Validators.required)
    });
  }

  ngOnInit() {
    this.configUrl = this.configService.config.url;
    this.getWorkflowSettings();
  }

  getWorkflowSettings(): void {
    this.workflowSettings$ = this.settingService.getWorkflowSettings().subscribe((wfs: Array<WorkflowSetting>) => {
      this.workflowSettings = wfs;
    });
  }

  updateQueueSettings(key: string, value: boolean): void {
    if (this.queueSetting.hasOwnProperty(key) && this.queueSetting[key] !== value) {
      this.alertService.info('Updating settings');
      this.queueSetting[key] = value;
      this.settingService.setQueueSettings(this.queueSetting).subscribe((s: QueueSetting) => {
          if (s) {
            this.queueSetting = s;
            this.alertService.success('Settings Updated');
            // should we use an output to let the parent know we updated so it can pull the groups again?
          } else {
            this.alertService.error('Setting could not be updated');
          }
        },
        error => {
          this.alertService.error('Settings could not be updated');
        })
    }
  }

  createFile(): void {
    if (this.newFileForm.valid) {
      this.alertService.info('Creating file for ' + this.queueSetting.name);
      this.settingService.createFile(this.queueSetting.name, this.newFileForm.get('name').value).subscribe(response => {
        this.alertService.success('File created for ' + this.queueSetting.name);
        this.getWorkflowSettings();
        this.newFileForm.reset();
      });
    } else {
      this.alertService.error('File could not be created for ' + this.queueSetting.name);
    }
  }

  deleteFile(filename: string) {
    let confirmed = confirm('Are you sure you want to delete ' + filename + ' from the ' + this.queueSetting.name + ' queue?');
    if (confirmed) {
      this.alertService.info(`Deleting "${filename}" from the ${this.queueSetting.name} queue`);
      this.settingService.deleteFile(this.queueSetting.name, filename).subscribe(response => {
        this.alertService.success(`Deleted "${filename}" from the ${this.queueSetting.name} queue`);
        this.getWorkflowSettings();
      });
    } else {
      this.alertService.error(`Canceled delete of "${filename}" from the ${this.queueSetting.name} queue`);
    }
  }

  ngOnDestroy(): void {
    if (this.workflowSettings$) {
      this.workflowSettings$.unsubscribe();
    }
  }

}
